require 'default:spellbook_dynamic_window'

-- variables used in the default namespace
mPageType = "PrestigeIndex"
mBookType = nil

-- place to hold all of our custom combat book profiles
CombatbookProfile = {}

-- generic profile to inherit from change as needed
CombatbookProfile.GenericBook =
{
	-- the Key in your PrestigeData table
	PrestigeKey = "Edit Me",
	-- storing user interface element information
	UIElementText =
	{
		-- title information for the page
		IndexPageTitle = "Edit Me",
	},
	-- storing user interface element information
	UIElementColor = 
	{
		AbilityDetailHeader = "FFFFFF", 
		AbilityDetailHeaderText = "777777",
		IndexDivider = "FFFFFF",
		IndexTitleText = "777777",
	},	
}

---------------------------------------------------------------------------------
-- BEGIN COMBAT PROFILES HERE
---------------------------------------------------------------------------------
-- DIRTY FIGHTING
-- copy the generic table over
-- when choosing the Key name make sure its the same as the combatbook objects
-- variable BookType. so in this example our BookType is DirtyFighting
CombatbookProfile.DirtyFighting = CombatbookProfile.GenericBook
-- set our PrestigeData table key
CombatbookProfile.DirtyFighting.PrestigeKey = "DirtyFighting"
-- set the title of the index page when you open it
CombatbookProfile.DirtyFighting.UIElementText.IndexPageTitle = "Dirty Fighting"

-- YOUR PROFILES

---------------------------------------------------------------------------------
-- END OF COMBATBOOK PROFILES HERE
---------------------------------------------------------------------------------

--- handles the dynamic window response events
-- @param user (PlayerObject) - always throwing players through parameters
-- @param returnId (string) - a string sent from the button response
function HandleDynamicWindowResponse(user, returnId)
	if(returnId ~= nil) then
		local result = StringSplit(returnId, "|")
		if(result[1] == "ChangePage") then			
			mPageType = result[2]
			mPageNumber = tonumber(result[3])
			ShowCustomCombatbookDialog()					
			return
		end
	end
	this:DelModule("spellbook_dynamic_window")
end

--- handles the open combat book message event
-- @param combatBook (GameObject) - any game object but prefered combatbooks
-- @param pageType (string) - handles page showing the combat ability index or details
function HandleOpenCombatBook(combatBook,pageType)
	if ( combatBook == nil ) then return end
	if(pageType) then
		mPageType = pageType
		mPageNumber = 1
	end
	if(mBookType == nil) then
		mBookType = combatBook:GetObjVar("BookType")
	end
	ShowCustomCombatbookDialog()	
end

--- function to display the prestige abilities
-- @param dynamicWindow (DynamicWindow) - dynamicwindow object to attach too
-- @param abilityKey (string) - they KEY string in the abilities table
-- @param xOffset (number) - good starting point for x
-- @param abilityType (string) - currently isn't used by citadel but keeping it here in case of future updates
function ShowAbilityDetail(dynamicWindow,abilityKey,xOffset,abilityType)
	local actionData = GetPrestigeAbilityUserAction(this, nil,CombatbookProfile[mBookType].PrestigeKey,abilityKey)
	local abilityData = PrestigeData[CombatbookProfile[mBookType].PrestigeKey].Abilities[abilityKey]
	local descStr = abilityData.Tooltip .. "\n\n".. "Difficulty: "..abilityData.Rank or 1
	dynamicWindow:AddUserAction(xOffset+208,60,actionData,128,128)
	dynamicWindow:AddImage(xOffset + 146,138,"Prestige_TitleHeader",191,28,"Sliced",CombatbookProfile[mBookType].UIElementColor.AbilityDetailHeader)
	dynamicWindow:AddLabel(xOffset + 240,141,"[" .. CombatbookProfile[mBookType].UIElementColor.AbilityDetailHeaderText .. "]"..(abilityData.Action.DisplayName).."[-]",150,0,28,"center",false,false,"Kingthings_Dynamic")
	dynamicWindow:AddImage(xOffset + 114,174,"Prestige_Divider",250,0,"Sliced",CombatbookProfile[mBookType].UIElementColor.AbilityDetailHeader)
	dynamicWindow:AddLabel(xOffset + 114,192,"[43240f]"..descStr.."[-]",220,300,18,"left",false,false,"PermianSlabSerif_Dynamic_Bold")
end

--- display the custom combat dialog book
function ShowCustomCombatbookDialog()
	local detailPageStr = "PrestigeDetail"
	local isDetailPage = (mPageType == "PrestigeDetail")
	local counts = { PrestigeIndex = 0 }
	local prestigeCount = 0

	for i,entry in pairs(PrestigeData[CombatbookProfile[mBookType].PrestigeKey].Abilities) do
		if ( entry.AbilityType == CombatbookProfile[mBookType].PrestigeKey ) then
			prestigeCount = prestigeCount + 1
		end
	end
	counts.PrestigeIndex = prestigeCount
	counts.PrestigeDetail = prestigeCount
	local dynamicWindow = DynamicWindow(SPELLBOOK_DIALOG,"",SPELLBOOK_WIDTH,SPELLBOOK_HEIGHT,-SPELLBOOK_WIDTH/2,-SPELLBOOK_HEIGHT/2,"TransparentDraggable","Center")
	dynamicWindow:AddImage(0,0,"Spellbook",SPELLBOOK_WIDTH,SPELLBOOK_HEIGHT)
	if(isDetailPage) then		
		local isFirstPage = (mPageNumber == 1)
		local isLastPage = (mPageNumber == math.ceil(counts[mPageType]/2))

		if not(isFirstPage) then
			local pageStr = tostring(mPageNumber - 1)
			dynamicWindow:AddButton(60,24,"ChangePage|"..detailPageStr.."|"..pageStr,"",154,93,"","",false,"BookPageDown")
		end

		if not(isLastPage) then
			local pageStr = tostring(mPageNumber + 1)
			dynamicWindow:AddButton(574,24,"ChangePage|"..detailPageStr.."|"..pageStr,"",154,93,"","",false,"BookPageUp")
		end
	end
	dynamicWindow:AddButton(726,22,"","",0,0,"","",true,"CloseSquare",buttonState)
	dynamicWindow:AddButton(3,70,"ChangePage|PrestigeIndex|1","",78,58,"","",false,"EvocationTab","pressed")
	if (mPageType == "PrestigeIndex") then
		local xOffset = 110
		local yOffset = 90
		local itemIndex = 1
		local sortedList = SortAbilityTable()
		dynamicWindow:AddLabel(236,44,"[" .. CombatbookProfile[mBookType].UIElementColor.IndexTitleText .. "]" .. CombatbookProfile[mBookType].UIElementText.IndexPageTitle .. "[-]",0,0,46,"center",false,false,"Kingthings_Calligraphica_Dynamic")
		dynamicWindow:AddImage(110,80, "SpellIndexInfo_Divider",250,0,"Sliced",CombatbookProfile[mBookType].UIElementColor.IndexDivider)	
		for i,entry in pairs(sortedList) do
			entry = entry.Data
			if(detailPageStr == "PrestigeDetail" and entry.AbilityType == CombatbookProfile[mBookType].PrestigeKey )  then 
				local pageStr = tostring(math.ceil(itemIndex/2))
				dynamicWindow:AddButton(xOffset,yOffset,"ChangePage|"..detailPageStr.."|"..pageStr,(entry.Rank or 1).."|"..(entry.Action.DisplayName),250,34,"","",false,"BookList")
				yOffset = yOffset + 36
				itemIndex = itemIndex + 1
				if(itemIndex == 9) then
					yOffset = 48
					xOffset = xOffset + 320
				end
			end
		end
	else
		local xOffset = 0
		local martialIndex = 1
		local martialStart = ((mPageNumber-1) * 2) + 1
		local sortedList = SortAbilityTable()
		for k,entry in pairs(sortedList) do
			local abilityName = entry.Name
			entry = entry.Data
			if(mPageType == "PrestigeDetail" and entry.AbilityType == CombatbookProfile[mBookType].PrestigeKey )  then 
				if(martialIndex >= martialStart and martialIndex <= martialStart + 1) then
					ShowAbilityDetail(dynamicWindow,abilityName,xOffset,PrestigeData[CombatbookProfile[mBookType].PrestigeKey])
					xOffset = 320
				end
				martialIndex = martialIndex + 1
			end
		end	
	end
	this:OpenDynamicWindow(dynamicWindow)
	mOpen = true
end

--- goes through the table data of combat abilities and sorts them alphabetically
function SortAbilityTable()
	local sortedAbilityList = {}
	for k,v in pairs(PrestigeData[CombatbookProfile[mBookType].PrestigeKey].Abilities) do
		table.insert(sortedAbilityList,{Name = k, Data = v})
	end
	table.sort(sortedAbilityList, function(a,b) 
		local RankA = (a.Data.Rank or 0)
		local RankB = (b.Data.Rank or 0)
		if(RankA == RankB) then
			return a.Data.Action.DisplayName < b.Data.Action.DisplayName
		else
			return RankA < RankB
		end
	end)
	return sortedAbilityList
end

--- this handles the dynamic window user responses
--- SPELLBOOK_DIALOG is from the default namespace
RegisterEventHandler(EventType.DynamicWindowResponse, SPELLBOOK_DIALOG, HandleDynamicWindowResponse)

-- this handles opening up the combat book
RegisterEventHandler(EventType.Message, "OpenSpellBook", HandleOpenCombatBook)