------------------------------------------------------------------------------------
-- INSTRUCTIONS
------------------------------------------------------------------------------------
-- To make any tool a elemental summoning tool, add the objvar ResourceSummoning to
-- the template of the tool.

-- require and override base_tool_resourceharvester
require 'default:base_tool_resourceharvester'

-- setting this to true means each time they use the harvested item, they lose 1 durability
-- setting this to false means use the standard durability loss
AlwaysLoseDurability = true

-- modify this to change the effect of the monsters appearing 
SpawnInEffect = "BuffEffect_C"

-- modify this to change the sound effect when the monster appears
SpawnInSound = "event:/character/combat_abilities/shockwave"

-- this table holds the resource types and spawn information for the elementals
-- Template: this is the template of the elemental to spawn
-- WarningMsg: this is the string to display to the player when the elemental Spawns
-- SpawnChance: this is the spawn chance between 0-1 (0%-100%)
ResourceElementalInfo =
{
	-- Mining
	Stone = { Template="stone_elemental", WarningMsg = "you summon forth a Stone Elemental", SpawnChance = 0.04 },
	IronOre = { Template="iron_elemental", WarningMsg = "you summon forth a Iron Elemental", SpawnChance = 0.04 },
	CopperOre = { Template="copper_elemental", WarningMsg = "you summon forth a Copper Elemental", SpawnChance = 0.03 },
	GoldOre = { Template="gold_elemental", WarningMsg = "you summon forth a Gold Elemental", SpawnChance = 0.03 },
	CobaltOre = { Template="cobalt_elemental", WarningMsg = "you summon forth a Cobalt Elemental", SpawnChance = 0.02 },
	ObsidianOre = { Template="obsidian_elemental", WarningMsg = "you summon forth a Obsidian Elemental", SpawnChance = 0.02 },

	-- Lumberjacking
	Wood = { Template="wood_elemental", WarningMsg = "you summon forth a Wood Elemental", SpawnChance = 0.04 },
	Ash = { Template="ash_elemental", WarningMsg = "you summon forth a Ash Elemental", SpawnChance = 0.03 },
	Blightwood = { Template="blightwood_elemental", WarningMsg = "you summon forth a Blightwood Elemental", SpawnChance = 0.02 },
}

--- remove a durability from the tool when used
--@ param user - player using the harvest tool
function HandleDurabilityLoss(user)
	local durability = this:GetObjVar("MaxDurability") or 0
	durability = durability - 1
	if(durability < 1) then
		this:Destroy()
		user:SystemMessage("your tool broke...")
	else
		print("durability loss..")
		this:SetObjVar("MaxDurability",durability)
		SetItemTooltip(this)
	end
end

--- spawn the resource elemental on top of the player
function SpawnResourceElemental(user,resourceType)
	CreateObj(ResourceElementalInfo[resourceType].Template,user:GetLoc(),"spawned_elemental",user)
	user:SystemMessage(ResourceElementalInfo[resourceType].WarningMsg,"event")
end

-- this second RegisterEventHandler will be called along side the original event handler
RegisterEventHandler(EventType.Message, "RequestResourceResponse", function(success,user,resourceType,objRef,countRemaining,failReason)
	-- are we in a guarded zone? if so return out
	if(GetGuardProtection(user) ~= "None") then return end

	-- if this isn't a summoning tool, skip
	if not(this:HasObjVar("ResourceSummoning")) then return end

	-- if the spawn was successful and it has a table entry spawn it
	if(success  and ResourceElementalInfo[resourceType] ~= nil) then
		if(AlwaysLoseDurability == true) then
			HandleDurabilityLoss(user)
		end
		if(math.random() <= ResourceElementalInfo[resourceType].SpawnChance) then
			SpawnResourceElemental(user,resourceType)
		end
	end
end)

RegisterEventHandler(EventType.CreatedObject,"spawned_elemental",function(success,obj,user)
	if(success) then
		obj:PlayEffect(SpawnInEffect)
		obj:PlayObjectSound(SpawnInSound)
	end
end)
