--- When crafting something out of a material (vs a resource) the resource get a weighted roll dependant on the type of material
--- Note: This is expected to be called BEFORE setting tooltips on the item!
-- @param objRef(GameObj) The recently created object.
-- @param material(string) The string name of the material the item was created from
-- @param skillLevel(number) The skill level of the skill used to create the item
-- @return none
function ApplyCraftedMaterialProperties(objRef, material, skillLevel)
	if ( objRef == nil ) then
		LuaDebugCallStack("[ApplyCraftedMaterialProperties] Nil objRef provided.")
		return
	end
	if ( Materials[material] == nil ) then
		LuaDebugCallStack("[ApplyCraftedMaterialProperties] Invalid material '"..tostring(material).."' provided.")
		return
	end
	if ( skillLevel == nil ) then
		LuaDebugCallStack("[ApplyCraftedMaterialProperties] Invalid skillLevel '"..tostring(skillLevel).."' provided.")
		return
	end
	local skillPercent = skillLevel / ServerSettings.Skills.PlayerSkillCap.Single

	-- pre-pend the name with the material
	local name = StripColorFromString(objRef:GetName())
	if ( ResourceData.ResourceInfo[material].CraftedItemPrefix ~= nil ) then
		objRef:SetName(MaterialTooltipColors[material]..ResourceData.ResourceInfo[material].CraftedItemPrefix.." "..name.."[-]")
	else
		objRef:SetName(name)
	end
	-- set the color from material
	objRef:SetHue(Materials[material])
	-- set an objvar of the type of material
	objRef:SetObjVar("Material", material)
	-- increase the max durability
	if ( ServerSettings.Crafting.MaterialBonus.Durability[material] > 0 ) then
		local durability = GetMaxDurabilityValue(objRef)
		-- add skill bonus
		--durability = durability + math.floor( ServerSettings.Crafting.SkillBonus.Durability.Max * skillPercent )
		objRef:SetObjVar("MaxDurability", durability + ServerSettings.Crafting.MaterialBonus.Durability[material])
	end

	-- add weapon bonuses
	if ( objRef:HasObjVar("WeaponType") ) then
		local attackBonus = WeightedRandom(0, ServerSettings.Crafting.NewSkillBonus.Attack.Max, ServerSettings.Crafting.NewSkillBonus.Attack.Weight(skillLevel))
		local accuracyBonus = WeightedRandom(0, ServerSettings.Crafting.NewSkillBonus.Accuracy.Max, ServerSettings.Crafting.NewSkillBonus.Accuracy.Weight(skillLevel))
		objRef:SetObjVar("AttackBonus", attackBonus)
		objRef:SetObjVar("AccuracyBonus", accuracyBonus)
	end

	-- add armor bonuses
	if ( objRef:HasObjVar("ArmorType") or objRef:HasObjVar("ShieldType") ) then
		local bonus = WeightedRandom(0, ServerSettings.Crafting.NewSkillBonus.Armor.Max, ServerSettings.Crafting.NewSkillBonus.Armor.Weight(skillLevel))
		objRef:SetObjVar("ArmorBonus", bonus)
	end

	-- add material bonus if its not a basic tier 1 material
	if(objRef:HasObjVar("ArmorType") or objRef:HasObjVar("ShieldType") or objRef:HasObjVar("WeaponType")) then
		-- does the material table exist?
		if(MaterialProperties[material]) then
			ApplyMaterialProperties(objRef,material,skillLevel)
			-- set the tooltip again on the item
			SetItemTooltip(objRef)
		end
	end
end

