
require 'default:globals.helpers.spells'
--- OVERRIDE ---
--- override is for adding a summoning skill check when casting
--- this function is called when the player casts a spell
-- @param spell - spell table key
-- @param mobileObj - object casting the spell
-- @param scrollObj - object of the scroll used to cast the spell
function CheckSpellCastSuccess(spell, mobileObj, scrollObj)
	if ( SpellData.AllSpells[spell] == nil or SpellData.AllSpells[spell].Skill == nil ) then return false end
	if( scrollObj ) then return true end
	-- mobs always succeed.
	if not( IsPlayerCharacter(mobileObj) ) then return true end
	-- Releasing a primed spell while stunned will result in cast failure.
	if ( HasMobileEffect(mobileObj, "Stun") ) then
		DoFizzle(mobileObj, "Cast failed.")
		return false
	end

	if ( SpellData.AllSpells[spell].Circle == nil ) then
		LuaDebugCallStack("Error: Circle is nil for spell: "..spell)
		return false
	end

	if ( SpellData.AllSpells[spell].Skill == nil ) then
		LuaDebugCallStack("Error: Skill is nil for spell: "..spell)
		return false
	end

	local circle = SpellData.AllSpells[spell].Circle
	local castSkillName = SpellData.AllSpells[spell].Skill

	-- if no scrollObj was passed
	if ( scrollObj == nil ) then
		circle = circle + 2 -- it's two circles easier to cast from scrolls
	end

	local skillDictionary = GetSkillDictionary(mobileObj)

	local skillLevel = GetSkillLevel(mobileObj, castSkillName, skillDictionary) or 0
	local skillGainMinMax = SkillValueMinMax(skillLevel, SpellData.CircleSkills[circle][1], SpellData.CircleSkills[circle][2])
	local success = Success( skillGainMinMax )

	--- when scroll casting, skill check only on a success
	if ( scrollObj == nil or success ) then

		-- only beneficial spells can gain channeling with casting
		if ( SpellData.AllSpells[spell].BeneficialSpellType == true ) then
			local channelingSkillLevel = GetSkillLevel(mobileObj, "ChannelingSkill", skillDictionary) or 0
			CheckSkillChance(
				mobileObj,
				"ChannelingSkill",
				channelingSkillLevel,
				SkillValueMinMax(channelingSkillLevel, SpellData.CircleSkills[circle][1], SpellData.CircleSkills[circle][2])
			)

		-- OVERRIDE START			
		elseif(SpellData.AllSpells[spell].AttackSpellType == true) then
			-- non-bene spells can gain affinity with casting
			local affinitySkillLevel = GetSkillLevel(mobileObj, "MagicAffinitySkill", skillDictionary) or 0
			CheckSkillChance(
				mobileObj,
				"MagicAffinitySkill",
				affinitySkillLevel,
				SkillValueMinMax(affinitySkillLevel, SpellData.CircleSkills[circle][1], SpellData.CircleSkills[circle][2])
			)
		elseif(SpellData.AllSpells[spell].IsSummoningSpell == true) then
			local summoningSkillLevel = GetSkillLevel(mobileObj, "SummoningSkill", skillDictionary) or 0
			CheckSkillChance(mobileObj,"SummoningSkill",GetSkillLevel(mobileObj,"SummoningSkill"),0.5)
		-- OVERRIDE END	

		end

		local rightHand = mobileObj:GetEquippedObject("RightHand")
		if ( rightHand ) then
			local type = rightHand:GetObjVar("WeaponType")
			if ( type and (type == "MagicStaff" or type == "MagicWand") ) then
				local attunementSkillLevel = GetSkillLevel(mobileObj, "MagicalAttunementSkill", skillDictionary) or 0
					CheckSkillChance(
						mobileObj,
						"MagicalAttunementSkill",
						attunementSkillLevel,
						SkillValueMinMax(attunementSkillLevel, SpellData.CircleSkills[circle][1], SpellData.CircleSkills[circle][2])
					)
			end
		end

		CheckSkillChance(mobileObj, castSkillName, skillLevel, skillGainMinMax)
	end
	
	-- only consume scroll on success
	if ( success and scrollObj ~= nil ) then
		-- consume the scroll, altering success along the way
		local resourceType = scrollObj:GetObjVar("ResourceType")
		-- TODO: MOVE STACKABLE FUNCTIONS TO GLOBALS SO WE CAN JUST ADJUST STACK DIRECTLY ON THE SCROLL OBJ
		success = ( scrollObj:IsValid() and scrollObj:GetObjVar("Spell") == spell 
				and resourceType and ConsumeResourceContainer(mobileObj, resourceType) )
	end

	if not( success ) then
		DoFizzle(mobileObj, "Cast failed.")
	end

	return success
end
