--- these variables are used by the summoning table system
local SummonCategoryKey = "NecromancySkill"
local SummonMobTableKey = "Liches"
local SummonDurationBase = 45

--- these table entries are the base level of the summoned lich
local SummonedTemplates = {}
SummonedTemplates[1] = "summoned_lich1" -- rank 1
SummonedTemplates[2] = "summoned_lich2" -- rank 2
SummonedTemplates[3] = "summoned_lich3" -- rank 3


--- standard citadel check for validation of the target
-- @pararm targetLoc - the location where the player targeted to summon this phantom
local function ValidateSpellTarget(targetLoc)
	-- is this location passable
	if( not(IsPassable(targetLoc)) ) then
		this:SystemMessage("[$2614]","info")
		return false
	end
	-- do we have line of sight
	if not(this:HasLineOfSightToLoc(targetLoc,ServerSettings.Combat.LOSEyeLevel)) then
		this:SystemMessage("[$2615]","info")
		return false
	end
	return true
end

--- this function ends the summoning process and removes this module
function EndEffect()
	this:DelModule("sp_summon_lich")
end

--- this handles the event message sent to this player to handle the target result
-- @param targetLoc - the location the player targeted sent by the engine
-- @param skipRemove - unused in this function
function HandleSpellTargetResult(targetLoc, skipRemove)
	-- validate the spell target location	
	if not(ValidateSpellTarget(targetLoc)) then
		EndEffect()
		return
	end
	-- apply a void pillar effect at the summon target location
	PlayEffectAtLoc("VoidPillar",targetLoc, 3)
	-- if we can create a summon, let us
	if(CanAddSummon(this,"NecromancySkill","Liches")) then
		CreateObj(SelectTamplate(), targetLoc, "LichCreated")
	end
end

--- this handles the event creation of the summoned object
-- @param success - true if the monster is created, false if not
-- @param objref - the phantom object that was created
function LichCreated(success, objref)
	if(success) then
		local necromancySkill = GetSkillLevel(this,"NecromancySkill")
		-- lets add addition duration to the summoned based on skill level
		-- duration bonus is between 0-50 based on necromancy skill
		local durationBonus = math.floor(necromancySkill/2)
		local duration = TimeSpan.FromSeconds(SummonDurationBase+durationBonus)
		
		-- add summon information to object
		objref:SetObjVar("Summoned",true)
		objref:SetObjVar("SummonMaster",this)
		-- attach the delete timer for summoned objects
		objref:AddModule("summoning.summon_delete_timer",{Duration=duration})
		-- key information for the summoning system
		objref:SetObjVar("CategoryKey",SummonCategoryKey)
		objref:SetObjVar("MobileTableKey",SummonMobTableKey)
		-- insert this mobile in to the players summon table object variable
		PlayerAddSummon(this,SummonCategoryKey,SummonMobTableKey,objref)
		-- play sound effect when summoned
		this:PlayObjectSound("event:/magic/air/magic_air_cast_air",false,0.7)
		-- now that everything is done, lets end this effect
		EndEffect()		
	end
end

--- this function will determine the template to spawn based on the
--- players necromancy skill and summoning skill
function SelectTamplate()
	local necromancySkill = GetSkillLevel(this,"NecromancySkill")	
	local summoningSkill = GetSkillLevel(this,"SummoningSkill")
	-- choice starts as rank 1
	local templateChoice = 1
	-- the summoning skill can add up to 2 in rank at 50 and 100 skill
	templateChoice = templateChoice + math.floor(summoningSkill/50)
	-- lets cap the template choice at 5 since our table only contains 5 entries
	if(templateChoice > 3) then templateChoice = 3 end
	-- return the correct template to spawn
	return SummonedTemplates[templateChoice]
end

-- fires when the spell target result message is sent to this module
RegisterEventHandler(EventType.Message,"SummonLichSpellTargetResult", HandleSpellTargetResult)

-- fires when the create object function is finished
RegisterEventHandler(EventType.CreatedObject, "LichCreated", LichCreated)
