require 'default:player'

--- SUMMONING RULES
--- in order to make it simple, when a player logs out their summons will instantly destroy
--- this will be handled by "Connect" and "Disconnect" message be passed to the event handlers
--- when the player logs in, their summon table object variable is refreshed and empty.

--- this handles the event for when this player changes worlds or connects
-- @param loginType - (string)the type of log out "Connect" or "ChangeWorld"
function HandleUserLoginEvent(loginType)
	-- if the player is logging in and not changing regions
	-- then we want to cleanup and initialize a new summons table
	if(loginType=="Connect") then
		-- if we don't have the SummonTable then lets set up the table objvar
		-- this will take care of setting up the summontable
		if not(this:HasObjVar("SummonTable")) then
			this:SetObjVar("SummonTable",{})
		end
		-- refresh the summon tables
		InitializePlayerSummonTable(this)
	end
end

--- this handles the event for when this player changes worlds or disconnects
-- @param logoutType - (string)the type of log out "Disconnect" or "ChangeWorld"
function HandleUserLogoutEvent(logoutType)
	-- if the player is logging out and not changing regeions
	-- then we want to destroy all their pesky summons.
	if(logoutType =="Disconnect") then
		-- destroys all active summons from this player
		-- then initializes a new summon table
		DestroyAllPlayerSummons(this)
	end
end

-- fires when a player logs in to your world
RegisterEventHandler(EventType.UserLogin,"",HandleUserLoginEvent)

-- fires when a player logs out of your world
RegisterEventHandler(EventType.UserLogout,"",HandleUserLogoutEvent)

